/* ---------------------------------------------------------------------- *
 * firestation.c
 * This file is part of lincity.
 * Lincity is copyright (c) I J Peters 1995-1997, (c) Greg Sharp 1997-2001.
 * (c) Corey Keasling, 2004
 * ---------------------------------------------------------------------- */


#include "firestation.h"


// FireStation:
FireStationConstructionGroup fireStationConstructionGroup(
    "Fire station",
    FALSE,                     /* need credit? */
    GROUP_FIRESTATION,
    GROUP_FIRESTATION_SIZE,
    GROUP_FIRESTATION_COLOUR,
    GROUP_FIRESTATION_COST_MUL,
    GROUP_FIRESTATION_BUL_COST,
    GROUP_FIRESTATION_FIREC,
    GROUP_FIRESTATION_COST,
    GROUP_FIRESTATION_TECH,
    GROUP_FIRESTATION_RANGE
);

Construction *FireStationConstructionGroup::createConstruction(int x, int y, unsigned short type) {
    return new FireStation(x, y, type);
}

void FireStation::update()
{
    ++daycount;
    if (commodityCount[STUFF_JOBS] >= FIRESTATION_JOBS
    &&  commodityCount[STUFF_GOODS] >= FIRESTATION_GOODS
    &&  commodityCount[STUFF_WASTE] + (FIRESTATION_GOODS / 3) <= MAX_WASTE_AT_FIRESTATION)
    {
        commodityCount[STUFF_JOBS] -= FIRESTATION_JOBS;
        commodityCount[STUFF_GOODS] -= FIRESTATION_GOODS;
        commodityCount[STUFF_WASTE] += (FIRESTATION_GOODS / 3);
        ++covercount;
        ++workingdays;
    }
    //monthly update
    if (total_time % 100 == 0)
    {
        busy = workingdays;
        workingdays = 0;
    }
    //animate
    if (animate && real_time > anim)
    {
        anim = real_time + FIRESTATION_ANIMATION_SPEED;
        switch (type)
        {
            case (CST_FIRESTATION_1):
                type = CST_FIRESTATION_2;
                break;
            case (CST_FIRESTATION_2):
                type = CST_FIRESTATION_3;
                break;
            case (CST_FIRESTATION_3):
                type = CST_FIRESTATION_4;
                break;
            case (CST_FIRESTATION_4):
                type = CST_FIRESTATION_5;
                break;
            case (CST_FIRESTATION_5):
                type = CST_FIRESTATION_6;
                break;
            case (CST_FIRESTATION_6):
                type = CST_FIRESTATION_7;
                anim += 10 * FIRESTATION_ANIMATION_SPEED; /* pause */
                break;
            case (CST_FIRESTATION_7):
                type = CST_FIRESTATION_8;
                break;
            case (CST_FIRESTATION_8):
                type = CST_FIRESTATION_9;
                break;
            case (CST_FIRESTATION_9):
                type = CST_FIRESTATION_10;
                break;
            case (CST_FIRESTATION_10):
                type = CST_FIRESTATION_1;
                animate = false;        /* stop */
                break;
        }
    }
    /* That's all. Cover is done by different functions every 3 months or so. */
    fire_cost += FIRESTATION_RUNNING_COST;
    if(refresh_cover)
    {   cover();}
}

void FireStation::cover()
{
    if(covercount < daycount)
    {
        daycount = 0;
        active = false;
        return;
    }
    active = true;
    covercount -= daycount;
    daycount = 0;
    animate = true;
    for(int yy = ys; yy < ye; ++yy)
    {
        for(int xx = xs; xx < xe; ++xx)
        {
            world(xx,yy)->flags |= FLAG_FIRE_COVER;
        }
    }
}

void FireStation::report()
{
    int i = 0;
    const char* p;
    mps_store_sd(i++,constructionGroup->name,ID);
    mps_store_sfp(i++, "busy", (float) busy);
    i++;
    list_commodities(&i);
    p = active?"Yes":"No";
    mps_store_ss(i++, "Fire Protection", p);
}

/** @file lincity/modules/firestation.cpp */

